/*
** Charge_Sound.c for Gmurf in ~/Gmurf/Charge_Sound/
** 
** Made by 
** Login   <ahes@ker-ys.epita.fr>
** 
** Started on  Fri Mar  5 19:03:30 1999 
** Last update Mon Mar 15 20:30:02 1999 Rmi Coupet
*/
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include "Errors.h"
#include <stdlib.h>
#include <unistd.h>
#include "Fct_Sys.h"
#include "DebugLib.h"
#include "WavePlay.h"
#include "Set_Rate.h"
#include "8bM_16bS.h"
#include <sys/wait.h>
#include "Swap_File.h"
#include "Constants.h"
#include "WaveHeader.h"
#include "Load_Sound.h"
#include "Parse_Loader_Conf.h"

/*
** new_tempgmurf_for_swap: make a new temp file
*/
static	t_tempgmurf*	new_tempgmurf_for_swap(name)
char*			name;
{
  t_tempgmurf* tempgmurf;

  tempgmurf = (t_tempgmurf *) malloc(sizeof(t_tempgmurf) + 200);
  tempgmurf->file = open(name, O_RDWR | O_CREAT | O_APPEND | O_TRUNC, 0644);
  if (tempgmurf->file == -1)
    {
      free(tempgmurf);
#ifdef TEXT_MSG
      printf("%s\n", BAD_TEMP);
      printf("errno : %d name : %s\n", errno, name) ;
      gmurf_errno = BAD_CREAT;
      warning("Charge_Sound.c: new_tempgmurf_for_swap: Error when opening the tempfile");
#endif
      return (NULL);
    }
  strcpy(tempgmurf->tempname,name);
  return (tempgmurf);
} /* new_tempgmurf */

/*
** exec_plug_in: ask the plug in path and execute it
*/
static	int	exec_plug_in(argv, file2load, plug_in_path)
char		*argv[3];
char		*file2load;
char		*plug_in_path;
{
  if (execlp(plug_in_path, plug_in_path, swap_file, NULL) == -1)
  {
#ifdef TEXT_MSG
    printf("error during the execution of %s with %s\n", argv[0], file2load);
    printf("errno = %d\n", errno);
#endif
    exit(-1);
    return (1);
  }
return (-1);
}

/*
** get_plug_in_result: get the result of the plug ins (the structure tempgmurf)
*/
static	t_tempgmurf	*get_plug_in_result(error_code)
int			error_code;
{
  t_tempgmurf	*temp_gmurf;

  if(WEXITSTATUS(error_code) != 1)
    {
#ifdef TEXT_MSG
      printf("error during the reading of the sound file\n");
#endif
      gmurf_errno = BAD_PLUGIN_EXEC;
      return (NULL);
    }
  temp_gmurf = (t_tempgmurf *) malloc(sizeof(t_tempgmurf));
  load_swap_file((void *) temp_gmurf, sizeof(t_tempgmurf));
  unlink(swap_file);
#ifdef TEXT_MSG
  printf("\ntemp_gmurf->tempname         = %s\n", temp_gmurf->tempname);
  printf("temp_gmurf->origname         = %s\n", temp_gmurf->origname);
  printf("temp_gmurf->file             = %d\n", temp_gmurf->file);
  printf("temp_gmurf->temp_size        = %d\n", temp_gmurf->temp_size);
  printf("temp_gmurf->bits_per_sample  = %d\n", temp_gmurf->bits_per_sample);
  printf("temp_gmurf->chanels          = %d\n", temp_gmurf->chanels);
  printf("temp_gmurf->sample_rate      = %d\n\n", (int) temp_gmurf->sample_rate);
#endif
  return (temp_gmurf);
}

/*
** charge_sound_file: load a sound file in a tempfile and save its attrubuts
*/
t_tempgmurf	*charge_sound_file(wav_path, temp_path, gmurf_rate)
char		*wav_path;
char		*temp_path;
int		gmurf_rate;
{
  int		child_status;
  char		*argv[3];
  char		*plug_in_path;
  PID_T		pid;
  t_tempgmurf	*temp_gmurf;

  if ((plug_in_path = get_sound_loader_path(wav_path)) == NULL)
    return(NULL);
  if ((temp_gmurf  = new_tempgmurf_for_swap(temp_path)) == NULL)
    return (NULL);
  strcpy(temp_gmurf->origname, wav_path);
  if (make_swap_file((void *) temp_gmurf, sizeof(t_tempgmurf)) == -1)
    return (NULL);
  free(temp_gmurf);
  if ((pid = fork()) == -1) 
    {
      gmurf_errno = BAD_FORK;
      return (NULL);
    }
  if (pid == 0)
    {
      if (exec_plug_in(argv, wav_path, plug_in_path) == -1)
	{
	  print_gmurf_error("\0");
	  exit(-1);
	}
    }
  else        
    {
      waitpid(pid, &child_status, WUNTRACED);
      temp_gmurf = get_plug_in_result(child_status);
    }
  if (temp_gmurf != NULL)
    {
      if ((temp_gmurf->bits_per_sample != 16) || (temp_gmurf->chanels != 2))
	convert2_16bits_stereo(temp_gmurf);
      if(temp_gmurf->sample_rate != gmurf_rate)
	set_rate(temp_gmurf, temp_gmurf->sample_rate, gmurf_rate);
    }
  return (temp_gmurf);
}
