/*
** Parse_Loader_Conf.c for Gmurf in ~/Gmurf/Charge_Sound/
** 
** Made by 
** Login   <ahes@ker-ys.epita.fr>
** 
** Started on  Fri Mar  5 14:46:48 1999 
** Last update Wed Mar 10 22:30:19 1999 
*/
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "Errors.h"
#include "Messages.h"
#include "Constants.h"
#include "Parse_Loader_Conf.h"

/*
** is_same_type: retourne 1 si 'file' est du type 'type' et 0 sinon 
*/
int	is_same_type(file, type)
char	*file;
char	*type;
{
  int	cpt;
  int	result;
  char	char_type;
  char	char_file;

  result = 1;
  for (cpt = 0; cpt < strlen(type); cpt++)
    {
      if ((type[cpt] >= 'a') && (type[cpt] <= 'z'))
	char_type = type[cpt] + 'A' - 'a';
      else
	char_type = type[cpt];
      if ((file[strlen(file) - strlen (type) + cpt] >= 'a') && (file[strlen(file) + strlen(type) + cpt] <= 'z'))
	char_file = file[strlen(file) - strlen(type) + cpt] + 'A' - 'a';
      else
	char_file = file[strlen(file) - strlen(type) + cpt];
      if (char_type != char_file)
	result = 0; 
    }
  return (result);
}

/*
** create_new_sound_file_loader: cree un nouveau fichier (vide) contenant les chemin des plug-ins
**				 relatifs a la lecture de fichiers sonores
*/

FILE	*create_new_sound_file_loader()
{
  FILE	*temp_file;

  printf("%s %s\n", FILE_NON_EXISTENT, SOUND_FILE_LOADER);
  printf("...%s\n", FILE_CREAT);
  if ((temp_file = fopen(SOUND_FILE_LOADER, "a+")) == NULL)
    printf("%s\n", BAD_CREAT);
  return (temp_file);
}

/*
** get_next_word: parse une chaine de caracteres et extrait le premier mot trouve
**		  (compris entre le debut de la ligne ou apres des espace jusqu'a l'espace suivant)
*/
char	*get_next_word(buf, cpt, line)
char	*buf;
int	*cpt;
int	line;
{
  int	end;
  int	begin;
  char	*result;

  while (((buf[*cpt] == ' ') || (buf[*cpt] == '\0')) && (*cpt < MAX_LINE_LENGTH)) 
    (*cpt)++;
  if ((*cpt == MAX_LINE_LENGTH) || (*cpt > strlen(buf)))
    {
      printf("%s:\n %s:\n   %s\n ligne %d :     %s\n", GMURF_ERROR, BAD_SCRIPT, SOUND_FILE_LOADER, line, buf);
      exit(-1);
    }
  begin = *cpt;
  while ((buf[*cpt] != ' ') && (buf[*cpt] != '\0') && (buf[*cpt] != '\n') && (*cpt < MAX_LINE_LENGTH))
    (*cpt)++;
  if ((*cpt == MAX_LINE_LENGTH))
    {
      printf("%s:\n %s:\n   %s\n %s %d: '%s'\n", GMURF_ERROR, BAD_SCRIPT, SOUND_FILE_LOADER, BAD_LINE, line, buf);
      exit(-1);
    }
  end = *cpt;
  result = (char *) malloc((end - begin) * sizeof(char));
  for (*cpt = begin; *cpt < end; ++(*cpt))
    result[*cpt - begin] = buf[*cpt];
  result[*cpt] = '\0';
  return (result);
}  

/*
** test_sound_file_loader: teste si le fichier contenant les chemins des plug-ins servant a lire des
**			   fichiers sonores est valide.
**			   Cette procedure quite Gmurf si le resultat de l'operation est negatif
*/
void	test_sound_file_loader()
{
  int	cpt;
  int	line;
  FILE	*file2test;
  char	buf[MAX_LINE_LENGTH];
  char	*path;
  
  if (access(SOUND_FILE_LOADER, F_OK) == -1)
    file2test = create_new_sound_file_loader();
  else
    {
      file2test = fopen(SOUND_FILE_LOADER, "r");
      line = 0;
      while (!feof(file2test))
	{
	  fgets(buf, MAX_LINE_LENGTH, file2test);
	  line++;
	  if ((buf[0] != '#') && (strlen(buf) >= 2))
	    {
	      cpt = 0;
	      free(get_next_word(buf, &cpt, line));
	      path = get_next_word(buf, &cpt, line);
	      if(access(path, F_OK | R_OK | X_OK) == -1)
		{
		  printf("%s: %s(%s)\n  %s %d:%s", GMURF_ERROR, BAD_ACCESS, path, BAD_LINE, line, buf);
		  exit(-1);
		}
	      free(path);
	    }
	}
    }
  if (file2test != NULL)
    fclose(file2test);
}

/*
** get_sound_loader_path: obtient le chemin du plug-ins servant a lire le fichier sonore 'file'
**			  en cas de reponse negative, retourne NULL
*/
char	*get_sound_loader_path(file)
char	*file;
{
  int	cpt;
  int	line;
  FILE	*pathfile;
  char	buf[MAX_LINE_LENGTH];
  char  *type;
  char  *path;

  pathfile = fopen(SOUND_FILE_LOADER, "r");
  line = 0;
  while (!feof(pathfile))
    {
      fgets(buf, MAX_LINE_LENGTH, pathfile);
      line++;
      if ((buf[0] != '#') && (strlen(buf) >= 2))
	{
	  cpt = 0;
	  type = get_next_word(buf, &cpt, line);
	  path = get_next_word(buf, &cpt, line);
	  if (is_same_type(file, type) == 1)
	    {
	      fclose(pathfile);
	      free(type);
	      return (path);
	    }
	  free(type); free(path);
	}
    }
  gmurf_errno = UNKNOWN_SOUND_TYPE;
  fclose(pathfile);
  return (NULL);
}









